IF OBJECT_ID('[Builds_Extended]') IS NOT NULL DROP VIEW [Builds_Extended]
GO

CREATE VIEW [Builds_Extended] AS

SELECT R.[Release_Id]
	  ,R.[Release_Number]
	  ,B.[Build_Id]
      ,B.[Build_Number]
      ,B.[BuildStatus_Name]
      ,[Release_Name] = COALESCE(R.[Release_Name], R.[Release_Number])
      ,[Target_Date] = RT_LATEST_END.[End_Date]
      ,[Earliest_Target_Start_Date] = RT_EARLIEST_START.[Start_Date]
      ,[Latest_Target_End_Date] = RT_LATEST_END.[End_Date]
	  ,B.[Build_Sequence]
      ,[Pipeline_Name] = COALESCE(B.[Pipeline_Name], R.[Pipeline_Name])
      ,R.[ReleaseStatus_Name]
      ,R.[Release_Sequence]
      ,[Release_Notes_Text] = R.[Notes_Text]
      ,B.[Rejection_Notes]
      ,B.[RejectedBy_User_Name]
      ,B.[Rejected_Date]
      ,B.[CreatedBy_User_Name]
      ,B.[CreatedOn_Date]
      ,B.[ModifiedBy_User_Name]
      ,B.[ModifiedOn_Date]

	  ,A.[Application_Id]
	  ,A.[Application_Name]

	  ,[Latest_Promotion_Id] = BP.[Promotion_Id]
	  ,[Latest_Promotion_Date] = BP.[Promoted_Date]
	  ,[Latest_Promotion_Status] = BP.[PromotionStatus_Name]
	  ,[Latest_PipelineStage_Name] = BP.[PipelineStage_Name]
	  ,B.[Furthest_PipelineStage_Name]

	  ,[Latest_Execution_Id] = BE.[Execution_Id]
	  ,[Latest_Execution_Environment_Id] = ENV.[Environment_Id]
	  ,[Latest_Execution_Environment_Name] = ENV.[Environment_Name]
	  ,[Latest_Execution_RunState_Code] = E.[ExecutionRunState_Code]
	  ,[Latest_Execution_Status_Code] = E.[ExecutionStatus_Code]

  FROM [Builds] B
       LEFT JOIN [Releases] R
               ON B.[Release_Id] = R.[Release_Id]
       LEFT JOIN [Applications] A
               ON B.[Application_Id] = A.[Application_Id]
        OUTER APPLY(
            SELECT TOP 1 * FROM [BuildPromotions] BP WHERE BP.[Build_Id] = B.[Build_Id] ORDER BY BP.[Promoted_Date] DESC
        ) BP
        OUTER APPLY(
            SELECT TOP 1 * FROM [PipelineStageTargetExecutions] BE WHERE BE.[Promotion_Id] = BP.[Promotion_Id] ORDER BY [Execution_Id] DESC
        ) BE
        OUTER APPLY(
            SELECT TOP 1 * FROM [ReleaseTargetDates] RTD WHERE RTD.[Release_Id] = R.[Release_Id] ORDER BY RTD.[Start_Date] ASC
        ) RT_EARLIEST_START
        OUTER APPLY(
            SELECT TOP 1 * FROM [ReleaseTargetDates] RTD WHERE RTD.[Release_Id] = R.[Release_Id] ORDER BY RTD.[End_Date] DESC
        ) RT_LATEST_END
		LEFT JOIN [Executions] E
		       ON E.[Execution_Id] = BE.[Execution_Id]
		LEFT JOIN [Environments] ENV
		       ON ENV.[Environment_Name] = BE.[Environment_Name]
GO
