﻿IF OBJECT_ID('[Executions_AddLogEntries]') IS NOT NULL DROP PROCEDURE [Executions_AddLogEntries]
GO

SET QUOTED_IDENTIFIER ON
GO

EXEC [__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Executions_AddLogEntries',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Writes multiple log entries.'
GO

IF TYPE_ID('ScopedExecutionLogEntry') IS NOT NULL DROP TYPE ScopedExecutionLogEntry

CREATE TYPE ScopedExecutionLogEntry AS TABLE
(
	[LogEntry_Sequence] INT NOT NULL,
	[LogEntry_Level] INT NOT NULL,
	[LogEntry_Text] VARBINARY(MAX) NOT NULL,
	[LogEntry_Date] DATETIME NOT NULL
)
GO

GRANT EXECUTE ON TYPE::[ScopedExecutionLogEntry] TO [BuildMasterUser_Role]
GO

CREATE PROCEDURE [Executions_AddLogEntries]
(
	@Execution_Id INT,
	@Scope_Sequence INT,
	@LogEntries_Table ScopedExecutionLogEntry READONLY,
	@Start_Sequence INT = NULL
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF @Start_Sequence IS NULL
		SET @Start_Sequence = (SELECT [Next_Sequence] = COALESCE(MAX([LogEntry_Sequence]), 0) + 1
	                             FROM [ScopedExecutionLogEntries]
	                            WHERE [Execution_Id] = @Execution_Id)

	INSERT INTO [ScopedExecutionLogEntries]
	(
		[Execution_Id],
		[Scope_Sequence],
		[LogEntry_Sequence],
		[LogEntry_Level],
		[LogEntry_Text],
		[LogEntry_Date]
	)
	SELECT @Execution_Id,
	       @Scope_Sequence,
		   @Start_Sequence + [LogEntry_Sequence],
		   [LogEntry_Level],
		   [LogEntry_Text],
		   [LogEntry_Date]
	  FROM @LogEntries_Table

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Executions_AddLogEntries] TO [BuildMasterUser_Role]
